/*
  Main program file for IRA Assignment 2.  You don't need to edit this file.
*/

#include <dolt-lite.h>
#include <stdio.h>
#include <vector>
#include "input.h"
#include "simrobot.h"
#include "localization.h"

using namespace std;
using namespace dolt;

/* global variables: */

Graphics g;			// dolt graphics object
Problem  prob;			// problem specification (parameters)
simRobot R;			// simulated robot
CommandList cmds;		// robot commands to execute
World world;


objectList world_list;	// you only need this for displaying the world
objectList locGraphics;

/********************************************************************/
/* the rest should be fine as-is, but feel free to change it        */
/********************************************************************/

/* keyboard handler */
void keyboard(unsigned char key)
{
  switch(key) {
  case 'q':
  case 'Q': // quit
    exit(0);

  case 'n': 
  case 'N':
    nextLocalizationStep();
    break;

  case 'w':
  case 'W': // show/hide the world
    world_list.toggle_visibility();
    break;
  case 'l':
  case 'L': // show/hide localization graphics
    locGraphics.toggle_visibility();
    break;

  case 'h':
  case 'H':
  default:
    printf("Commands:\n q: Quit\n h: Help\n"
	   " n: execute next robot command\n");
    printf("\n w or W: show/hide the World\n"
	   " l or L: show/hide the localization graphics\n");
    g.printControlKeyBindings();
    break;
  }
}

/* main program */
int main(int argc, char **argv)
{
  if(argc < 2) {
    printf("Usage: %s <problem file>\n", argv[0]);
    return 1;
  }

  // read the problem file and initialize the world list, simrobot,
  // and command list
  if(!prob.read_file(argv[1], world, R, cmds)) {
    return 1;
  }
  world_list.push_back(&world); // boundary and obstacles
  world_list.push_back(&R);	// simrobot

  double w = world.boundary().bbox().xmax() - world.boundary().bbox().xmin();
  double h = world.boundary().bbox().ymax() - world.boundary().bbox().ymin();
  unsigned int pw = 800, ph = 800;
  if(w > h)
    ph = (unsigned int)(800 * h / w);
  else
    pw = (unsigned int)(800 * w / h);

  g.initGL(&argc, argv, "IRA Assignment 1", pw, ph); // initialize dolt window
  g.setWorldBounds(world.boundary().bbox());
  g.setKeyboardHandler(keyboard); // register a keyboard callback function

  // tell dolt to draw the objects in each of these lists; the first
  // list is drawn on the bottom, the last on the top
  g.addObjectList(world_list);
  g.addObjectList(locGraphics);

  // make points a little bigger and green by default
  drawablePoint::setDefaultSize(6);
  drawablePoint::setDefaultObjectColor("green");

  initLocalization();
  g.runEventLoop(); // run the main loop

  return 0;
}
